#include "integrators.hpp"
#include "function.hpp"

// Integral calculation with Composite Quadratures
//-----------------------------------------------------------------------------

void usage();

int main (int argc, char* argv[])
    {

    cout << "Integral calculation with Composite Quadratures" << endl;
    cout << "V.2006-04-17 [" << LIB_ID << "], copyleft (c) 2006 Alexey Beshenov" << endl;
    cout << "This program is distributed under the terms of the GNU GPL, see gpl.txt" << endl << endl;

// number of arguments including keys, values, program name:
    int pn=11;

    if( argc != pn && argc != pn+2 ) { usage(); exit(EXIT_FAILURE); }

//-----------------------------------------------------------------------------

    int c, f, g, n, s; long double a, b;
    bool a_def=0, b_def=0, f_def=0, g_def=0, n_def=0, s_def=0;

    for (c=1; c<argc; c++) {
        if (!strcmp(argv[c], "-a")) {
            if (c<argc-1) { a = strtold(argv[c+1], NULL); a_def=1; }
            }
        if (!strcmp(argv[c], "-b")) {
            if (c<argc-1) { b = strtold(argv[c+1], NULL); b_def=1; }
            }
        if (!strcmp(argv[c], "-f")) {
            if (c<argc-1) { f = c+1; f_def=1; }
            }
        if (!strcmp(argv[c], "-g")) {
            if (c<argc-1) { g = c+1; g_def=1; }
            }
        if (!strcmp(argv[c], "-n")) {
            if (c<argc-1) { n = (int)strtold(argv[c+1], NULL); n_def=1; }
            }
        if (!strcmp(argv[c], "-s")) {
            if (c<argc-1) { s = (int)strtold(argv[c+1], NULL); s_def=1; }
            }
        }

//-----------------------------------------------------------------------------

    if( n< 1 || n > 10 ) {
        cerr << "ERROR! n must be an integer from 1 to 10!" << endl;
        exit(EXIT_FAILURE);
        }
    if( s < 1 ) {
        cerr << "ERROR! s must be an integer >= 1" << endl;
        exit(EXIT_FAILURE);
        }

    if (!a_def) { cerr << "ERROR! Missing parameter: -a" << endl; }
    if (!b_def) { cerr << "ERROR! Missing parameter: -b" << endl; }
    if (!f_def) { cerr << "ERROR! Missing parameter: -f" << endl; }
    if (!n_def) { cerr << "ERROR! Missing parameter: -n" << endl; }
    if (!s_def) { cerr << "ERROR! Missing parameter: -s" << endl; }
    if (!(a_def&&b_def&&f_def&&n_def&&s_def)) {
        usage(); exit(EXIT_FAILURE); cerr << endl;
        }

//-----------------------------------------------------------------------------

    if (!g_def) {                                 // function of real variable
        try { parser.Compile(addbrackets(argv[f])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        cout << "\\int_{" << a << "}^{" << b << "} [" << argv[f] << "]dx" << endl;
        cout.precision(OUTPUT_EPS); cout.setf(ios_base::fixed);
        cout << "~= " << chebyshev_calculate(n,a,b) << endl;
        }

    else {                                        // function of complex variable
        long double Re, Im;
        try { parser.Compile(addbrackets(argv[f])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        Re = composite_calculate(a,b,n,s);
        try { parser.Compile(addbrackets(argv[g])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        Im = composite_calculate(a,b,n,s);
        cout << "\\int_{" << a << "}^{" << b << "} [(" << argv[f] << ") + i*(" << argv[g] << ")]dx" << endl;
        ostringstream z;
        z.precision(OUTPUT_EPS); z.setf(ios_base::fixed);
        z << Re;
        if (fabs(Im) > EPS_LIM_T) { z << " + i*" << Im; }
        cout << "~= " << z.str() << endl;
        }

    PROC_TIME

        return 0;

    }


//-----------------------------------------------------------------------------

void usage()
    {
    cout << "Usage: -a <long double> -b <long double> -f \"<string>\" [-g \"<string>\"] -n <int> -s <int>" << endl << endl;
    cout << "[z(x)]dx, z(x) = f(x) + i*g(x) if -g defined (function of complex variable)" << endl;
    cout << "[f(x)]dx if -g not defined (function of real variable)" << endl;
    cout << "Interval: [a, b]" << endl;
    cout << "Method order: n, n <= 10" << endl;
    cout << "n = 1 -- Trapezoids Quadrature" << endl;
    cout << "n = 2 -- Simpson Quadrature" << endl;
    cout << "n = 3 -- Newton Quadrature" << endl;
    cout << "n = 4 -- Boole's rule" << endl;
    cout << "Iterations: s" << endl;
    cout << "[a, b] will be divided into s intervals" << endl;
    cout << "Use s = 1 for non-composite quadratures" << endl << endl;
    cout << builtin_functions;
    }
