#include "integrators.hpp"
#include "function.hpp"

// Fourier series
//-------------------------------------------------------------------

void usage();

int main (int argc, char* argv[])
    {

    cout << "Fourier series" << endl;
    cout << "V.2006-04-21 [" << LIB_ID << "], copyleft (c) 2006 Alexey Beshenov" << endl;
    cout << "This program is distributed under the terms of the GNU GPL, see gpl.txt" << endl << endl;

// number of arguments including keys, values, program name:
    int pn=9;

    if( argc != pn && argc != pn+2*2) { usage(); exit(EXIT_FAILURE); }

//-----------------------------------------------------------------------------

    int d, f, n; long double l, eps;
    bool l_def=0, f_def=0, n_def=0, eps_def=0, zl_def=0, zlf_def=0, zl=0, zlf=0;

    for (d=1; d<argc; d++) {
        if (!strcmp(argv[d], "-l")) {
            if (d<argc-1) { l = strtold(argv[d+1], NULL); l_def=1; }
            }
        if (!strcmp(argv[d], "-f")) {
            if (d<argc-1) { f = d+1; f_def=1; }
            }
        if (!strcmp(argv[d], "-n")) {
            if (d<argc-1) { n = (int)strtold(argv[d+1], NULL); n_def=1; }
            }
        if (!strcmp(argv[d], "-eps")) {
            if (d<argc-1) { eps = strtold(argv[d+1], NULL); eps_def=1; }
            }
        if (!strcmp(argv[d], "-zl")) {
            if (d<argc-1) { zl = (int)strtold(argv[d+1], NULL); zl_def=1; }
            }
        if (!strcmp(argv[d], "-zlf")) {
            if (d<argc-1) { zlf = (int)strtold(argv[d+1], NULL); zlf_def=1; }
            }
        }

//-----------------------------------------------------------------------------

    if (n < 1) {
        cerr << "ERROR! n must be an positive integer!" << endl;
        exit(EXIT_FAILURE);
        }

    if (zl != 0 && zl != 1) {
        cerr << "ERROR! zl must be an boolean!" << endl;
        exit(EXIT_FAILURE);
        }

    if (zlf != 0 && zlf != 1) {
        cerr << "ERROR! zlf must be an boolean!" << endl;
        exit(EXIT_FAILURE);
        }

    if (l <= 0) {
        cerr << "ERROR! l must be positive!" << endl;
        exit(EXIT_FAILURE);
        }

    if( LIMIT_EPS_T && eps < EPS_LIM_T ) {
        cout.setf(ios_base::scientific);
        cout << "ERROR! Can't work with precisions < " << EPS_LIM_T << endl;
        cout << "Using " << EPS_LIM_T << endl << endl;
        cout.unsetf(ios_base::scientific);
        eps = EPS_LIM_T;
        }

    if( LIMIT_EPS_B && eps > EPS_LIM_B ) {
        cout.setf(ios_base::scientific);
        cout << "ERROR! Can't work with precisions > " << EPS_LIM_B << endl;
        cout << "Using " << EPS_LIM_B << endl << endl;
        cout.unsetf(ios_base::scientific);
        eps = EPS_LIM_B;
        }

    if (!l_def) { cerr << "ERROR! Missing parameter: -l" << endl; }
    if (!f_def) { cerr << "ERROR! Missing parameter: -f" << endl; }
    if (!n_def) { cerr << "ERROR! Missing parameter: -n" << endl; }
    if (!eps_def) { cerr << "ERROR! Missing parameter: -eps" << endl; }
    if (!(zl_def==zlf_def)) { cerr << "ERROR! -zl and -zlf must be defined both" << endl; }
    if (!(l_def&&f_def&&n_def&&eps_def&&(zl_def==zlf_def))) {
        usage(); exit(EXIT_FAILURE); cerr << endl;
        }

//-------------------------------------------------------------------

    ld_array a; ld_array b;
// series coefficients

    long double c; int dlt_t, q, r, k;
    long double interval_a = -l, interval_b = l;

/* ZLF:
0 -- sine-series for (0, l)
1 -- cosine-series for (0, l) */

    if(zl == 1) { interval_a = 0; }

//-------------------------------------------------------------------

    for (k=0;k<=n;k++) {
        if (zl == 1 && zlf == 0) { a.push_back (0.0L); }
        else {
            ostringstream expression;

            expression << "(" << argv[f] << ")*" << DESIGNATION_COS << "(" << M_PI*(long double)k/l << "*x)";

            char *expression_cs = new char [(int)strlen(expression.str().c_str())+1];
            int zh = 0;
            for (; zh<(int)strlen(expression.str().c_str()); zh++) {
                expression_cs[zh] = expression.str().c_str()[zh];
                }
            expression_cs[zh++] = '\0';

            try { parser.Compile(addbrackets(expression_cs)); }
            catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }

            a.push_back (1.0L/l*PPA_calculate (interval_a, interval_b, eps*1e-1L, PPA_ALG));
            if (zl == 1) { a[k] *= 2.0L; }

            delete[] expression_cs;
            expression.clear();
            }

        }

//-------------------------------------------------------------------

    for (k=0;k<=n;k++) {
        if (zl == 1 && zlf == 1) { b.push_back (0.0L); }
        else {
            ostringstream expression;

            expression << "(" << argv[f] << ")*" << DESIGNATION_SIN << "(" << M_PI*(long double)k/l << "*x)";

            char *expression_cs = new char [(int)strlen(expression.str().c_str())+1];
            int zh = 0;
            for (; zh<(int)strlen(expression.str().c_str()); zh++) {
                expression_cs[zh] = expression.str().c_str()[zh];
                }
            expression_cs[zh++] = '\0';

            try { parser.Compile(addbrackets(expression_cs)); }
            catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }

            b.push_back (1.0L/l*PPA_calculate (interval_a, interval_b, eps*1e-1L, PPA_ALG));
            if (zl == 1) { b[k] *= 2.0L; }

            delete[] expression_cs;
            expression.clear();
            }
        }

//-------------------------------------------------------------------

    cout << argv[f] << " ~=" << endl;

    int precision = (int)floor(log10(eps));
    if (precision >= 0) { precision = 1; }
    else { precision *= -1; }

    cout.precision(precision); cout.setf(ios_base::fixed);

    if (fabs(a[0]) >= eps) { cout << a[0]/2.0; }

    for (k=1; k <= n; k++) {
        if (fabs(a[k]) >= eps) {
            if (a[k] > 0) { cout << " + "; };
            if (a[k] < 0) { cout << " - "; };
            cout << fabs(a[k]);
            cout << "*" << DESIGNATION_COS << "(" << M_PI*(long double)k/l << "*x)";
            }
        }

    for (k=1; k <= n; k++) {
        if (fabs(b[k]) >= eps) {
            if (b[k] > 0) { cout << " + "; };
            if (b[k] < 0) { cout << " - "; };
            cout << fabs(b[k]);
            if (M_PI*k/l != 0.0L) { cout << "*" << DESIGNATION_SIN << "(" << M_PI*(long double)k/l << "*x)"; }
            }
        }

    cout << endl;

//-------------------------------------------------------------------

    PROC_TIME

        return 0;

    }


//-----------------------------------------------------------------------------

void usage()
    {
    cout << "Usage: -l <long double> -f \"<string>\" -n <int> -eps <long double> [-zl <bool>  -zlf <bool>]" << endl << endl;
    cout << "Period: (-l, l) | (0, l) if zl = 1" << endl;
    cout << "zlf = 0: sine series for (0, l) | zlf = 1: cosine series for (0, l)" << endl << endl;
    cout << "Members in series: n" << endl;
    cout.setf(ios_base::scientific);
    cout << "Precision: eps (" << EPS_LIM_B << " -- " << EPS_LIM_T << ")" << endl;
    cout.unsetf(ios_base::scientific);
    cout << builtin_functions;
    }
