// Polynomial arithmetics.
//-------------------------------------------------------------------

long int factorial (long int n)
    {
    if (n == 0) {
        return 1;
        }
    else {
        long int p; long int result;
        for (p=1,result=1; p<=n; p++) {
            result *= p;
            }
        return result;
        }
    }


long int choose (long int n, long int k)
    {
    return factorial(n)/factorial(k)/factorial(n-k);
    }


//-------------------------------------------------------------------

// CODE BELOW RESERVED FOR FURTHER USE

//-------------------------------------------------------------------

long double polynomial_calculate(polynomial a, long double x)
    {
    int n; long double sum;
    for (n=0, sum=0; n<(int)a.size(); n++) {
        sum += a[n]*pow(x, (long double)(n));
        }
    return sum;
    }


//-------------------------------------------------------------------

polynomial polynomial_derivative(polynomial a)
    {
    int n; polynomial d;
    for (n=1; n<(int)a.size(); n++) {
        d.push_back (a[n]*n);
        }
    return d;
    }


//-------------------------------------------------------------------

polynomial polynomial_indefinite_integral(polynomial a)
    {
    int n; polynomial i;
    i.push_back (0.0L);                           // Integration constant
    for (n=0; n<(int)a.size(); n++) {
        i.push_back (a[n]/(n+1));
        }
    return i;
    }


//-------------------------------------------------------------------

polynomial polynomial_sum(polynomial a, polynomial b)
    {
    int n; polynomial sum;

    if (a.size() < b.size()) {
        while (a.size() < b.size()) { a.push_back(0.0L); }
        }
    if (b.size() < a.size()) {
        while (b.size() < a.size()) { b.push_back(0.0L); }
        }

    for (n=0; n<a.size(); n++) {
        sum.push_back (a[n]+b[n]);
        }
    return sum;
    }


//-------------------------------------------------------------------

polynomial polynomial_product(polynomial a, polynomial b)
    {
    int m, n; polynomial product;

    product.push_back (0.0L);

    for (m=0; m < a.size(); m++) {
        polynomial buff;

        for (n=0; n < m; n++) {
            buff.push_back (0.0L);
            }

        for (n=0; n < b.size(); n++) {
            buff.push_back (a[m]*b[n]);
            }

        product = polynomial_sum(product, buff);

        }

    return product;
    }


//-------------------------------------------------------------------

// Generator of Legendre polynomials
// P_n(x)=1/2^n \sum_{k=0}^[n/2] (-1)^k{n \choose k}{2(n-k) \choose n}x^{n-2k}

polynomial legendre_polynomial (int n)
    {
    int k; polynomial P;

    for (k=n/2; k>=0; k--) {

        if (n%2==1) { P.push_back (0.0L); }

        P.push_back (1.0L/pow(2.0L, (long double)n)*pow(-1.0L, k)*(choose(n,k)*choose(2*(n-k),n)));

        if (n%2==0 && k!=0) { P.push_back (0.0L); }

        }

    return P;

    }


//-------------------------------------------------------------------

char* polychar(polynomial a)
    {

    ostringstream result;
    result.setf(ios_base::showpos);

    int n;

    for (n=a.size()-1; n>=0; n--) {
        result << a[n];
        if (n!=0 && n!=1) {
            result.unsetf(ios_base::showpos);
            result << "*x^" << (long double)n;
            }
        if (n!=0 && n==1) {
            result.unsetf(ios_base::showpos);
            result << "*x";
            }

        result.setf(ios_base::showpos);
        }

    char *result_cs = new char [(int)strlen(result.str().c_str())+1];
    int zh = 0;
    for (; zh<(int)strlen(result.str().c_str()); zh++) {
        result_cs[zh] = result.str().c_str()[zh];
        }
    result_cs[zh++] = '\0';

    return result_cs;
    }


//-------------------------------------------------------------------

// Interpolation polynomial (Lagrange form)
char* interpolate(ld_array x, ld_array y)
    {

    int n = (int)x.size();

    int i; int j; long double product;
    polynomial sum;
    sum.push_back(0.0L);

    for (i=0; i<n; i++) {

        polynomial result;

        for (j=0, product=1.0L; j<n; j++) {
            if (j!=i) {
                product*=(x[i]-x[j]);
                if (x[i]==x[j]) {
                    cerr << "You can't set points with the same abscissas (" << x[i] << "," << x[j] << ")" << endl;
                    exit(EXIT_FAILURE);
                    }
                }
            }

        result.push_back (y[i]/product);

        for(j=0; j<n; j++) {
            if (j!=i) {
                polynomial buff;
                buff.push_back((-1.0L)*x[j]);
                buff.push_back(1.0L);
                result = polynomial_product(result, buff);
                }
            }

        sum = polynomial_sum(sum, result);

        }

    return polychar(sum);

    }


//-------------------------------------------------------------------
